<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <title>{{config.title}}</title>
    <link rel="stylesheet" type="text/css" href="{{ url_for('PWebSwaggerSystem.static', filename='css/swagger-ui.css') }}" />
    <style>
      html
      {
        box-sizing: border-box;
        overflow: -moz-scrollbars-vertical;
        overflow-y: scroll;
      }

      *,
      *:before,
      *:after
      {
        box-sizing: inherit;
      }

      body
      {
        margin:0;
        background: #fafafa;
      }
    </style>
  </head>

  <body>
    <div id="swagger-ui"></div>

    <script src="{{ url_for('PWebSwaggerSystem.static', filename='js/swagger-ui-bundle.js') }}" charset="UTF-8"> </script>
    <script src="{{ url_for('PWebSwaggerSystem.static', filename='js/swagger-ui-standalone-preset.js') }}" charset="UTF-8"> </script>
    <script>
        window.onload = function () {
            const ui = SwaggerUIBundle({
                url: "{{ url_for('PWebSwaggerSystem.pweb-swagger-json') }}",
                dom_id: '#swagger-ui',
                deepLinking: true,
                presets: [
                    SwaggerUIBundle.presets.apis,
                    SwaggerUIStandalonePreset
                ],
                plugins: [
                    SwaggerUIBundle.plugins.DownloadUrl
                ]
            });
            window.ui = ui;
        };
  </script>
  </body>
</html>